/*
	File:		TSpriteCollection.h

	Contains:	This class represents a group of related Sprites.  For example, all of the shots
				fired by a sprite.  In general, the various things that happen to sprites (moving,
				drawing, hit testing) all happen to a whole group at a time.

	Written by: Timothy Carroll	

	Copyright:	Copyright  1996-1999 by Apple Computer, Inc., All Rights Reserved.

				You may incorporate this Apple sample source code into your program(s) without
				restriction. This Apple sample source code has been provided "AS IS" and the
				responsibility for its operation is yours. You are not permitted to redistribute
				this Apple sample source code as "Apple sample source code" after having made
				changes. If you're going to re-distribute the source, we require that you make
				it clear in the source that the code was descended from Apple sample source
				code, but that you've made changes.

	Change History (most recent first):
				7/2/1999	Karl Groethe	Updated for Metrowerks Codewarror Pro 2.1
				
				8/15/96		Timothy Carroll	Initial Release
				

*/


#ifndef _TSpriteCollection_
#define _TSpriteCollection_

#pragma once

#include "TSprite.h"


#if PRAGMA_STRUCT_ALIGN
#pragma options align=power
#endif

class TSpriteCollection
{
	public:
		TSpriteCollection();
		~TSpriteCollection();
		void AddSprite (TSprite *theSprite);
		void RemoveSprite (TSprite *theSprite);
		
		void ProcessSpriteGroup (void);
		void HitTest (TSpriteCollection *targetGroup);
		void DrawSpriteGroup (void);
		
	protected:
		TSprite *fSpriteListHead;
};

#if PRAGMA_STRUCT_ALIGN
#pragma options align=reset
#endif

#endif /* _TSpriteCollection_ */